/**
 * Clase que define el comportamiento del bloque.
 * @author
 */
import java.util.*;
public class Block extends PhysicsElement  {
   private static int id=0; // Identificador inicial en cero.
   private final float mass; // masa del bloque, no puede cambiar.
   protected Vector2D pos_t; // vector de posicion del bloque.
   protected Vector2D pos_tPlusDelta; // vector de posicion en el delta tiempo transcurrido.
   private Vector2D pos_tPlusDelta1; // vector de posicion en el delta tiempo posterior para el calculo del estado futuro.
   protected Vector2D speed_t,speed_t1; // vector de velocidad del bloque.
   protected Vector2D speed_tPlusDelta; // vector de velocidad en el delta tiempo transcurrido.
   private Vector2D speed_tPlusDelta1; // vector de velocidad en el delta tiempo del estado futuro.
   private Vector<Spring> springs;  // This Vector is a growable array of objects
                            // it has nothing to do with Vector2D.
   private Vector<MyString> strings; // Vector de los elementos del tipo cuerda.
   private boolean fixed; // indicador de si esta fijo el bloque.
   private Vector2D force = new Vector2D(); // Vector de la fuerza neta aplicada al bloque.
   private Vector2D a; // vector de aceleracion del bloque.
   private Vector2D g; // vector de gravedad que afecta al bloque.

   /**
    * Crea elemento Bloque, inicializandolo en masa cero (0) y asignandole un numero de identificacion inico para los elementos fisicos.
    */
   private Block(){ // nobody can create a block without state
      super(id++);
      mass=0;
   }  
   /**
    * Constructor del elemento Bloque con su masa, posicion y velocidad al momento de su creacion.
    * @param mass   flotante para la asignacion de la masa del bloque.
    * @param position   vector de posicion en 2D del bloque.
    * @param speed  vector de velocidad del bloque.
    * @param fixed  indicador si es un bloque fijo.
    * @since
    */
   public Block(float mass, Vector2D position, Vector2D speed, boolean fixed){
      super(id++);
      this.mass = mass;
      pos_t = position;
      pos_tPlusDelta = position;
      this.speed_t = speed;
      speed_tPlusDelta = speed;
      springs = new Vector<Spring>();
      strings = new Vector<MyString>();
      this.fixed = fixed;
   }
   /**
    * Agrega un extremo del resorte al bloque.
    * @param spring resorte que se une al bloque.
    */
   public void attachSpring (Spring spring) {
     springs.add(spring);
   }
    /**
    * Remueve un extremo de un resorte al bloque.
    * @param spring resorte que se desvincula al bloque.
    */
   public void detachSpring (Spring spring) {
     springs.remove(spring);
   }
   /**
    * Agrega un extremo de la cuerda al bloque.
    * @param string extremo de la cuerda que se vincula al bloque.
    */
   public void attachString (MyString string) {
     strings.add(string);
   }
   /**
    * Remueve un extremo de la cuerda al bloque.
    * @param string cuerda que se desvincula al bloque
    */
   public void detachString (MyString string) {
     strings.remove(string);
   }
    /**
    * Obtiene posicion actual del bloque.
    * @return posicion del bloque
    */
   public Vector2D getPosition() {
      return pos_t;
   }
    /**
    * Obtiene posicion siguente del bloque, valido entre computeNextState y  updateState.
    * @return posicion del bloque despues del siguente delta
    */
   public Vector2D nextPosition() {
      return pos_tPlusDelta;
   }
   /**
    * Metodo de calculo del proximo estado de la fuerza neta del bloque.
    * @param delta_t tiempo de observacion entre estados.
    * @param gravity variable con el valor de la aceleracion de gravedad.
    */
   public void computeNextState(double delta_t, double gravity) {
       if(!fixed){
         force = new Vector2D();
         a = new Vector2D();
         g = new Vector2D(0,gravity);
         Spring spring;
         MyString string;

         for (int i=0; i<springs.size(); i++) {
            spring = springs.get(i);
            if (spring != null) force = force.plus(spring.getForce(this));
         }
         for (int i=0; i<strings.size(); i++) {
            string = strings.get(i);
            if (string != null) force = force.plus(string.getForce(this));
         }
         a = force.times(1/mass).plus(g);
         speed_tPlusDelta = speed_t.plus(a.times(delta_t));
         pos_tPlusDelta = pos_t.plus(speed_t.times(delta_t)).plus(a.times(delta_t*delta_t/2));
         }
   }
    /**
    * Metodo de actualizacion de la posicion del bloque, dependiendo si esta fijo o no.
    * @param damp incida presencia de amortiguacion.
    */
   public void updateState(boolean damp){
       if(!fixed){
           if(damp){
            speed_t = speed_tPlusDelta1;
            pos_t = pos_tPlusDelta1;
           }
           else{
            speed_t = speed_tPlusDelta;
            pos_t = pos_tPlusDelta;
           }
       }
   }
   /**
    * Metodo de chequeo del estado siguiente para la amortiguacion, diminuye la velocidad si es necesario.
    * @param delta_t tiempo de simulacion futura.
    */
   public void checkNext(double delta_t){
       MyString string;
       double amort = 0.6;
       double a1;
       if(!fixed){
           if (strings.size() == 0){
                speed_tPlusDelta1 = speed_tPlusDelta;
                pos_tPlusDelta1 = pos_tPlusDelta;
           }
           for (int i=0; i<strings.size(); i++) {
                string = strings.get(i);
                if (string != null){
                    if (string.getStretch() > 0 && string.nextStretch() < 0 ){
                            a1 = (string.getStretch())/delta_t;
                            speed_t1 = speed_t.minus(force.unitary().times(a1*amort));
                            speed_tPlusDelta1 = speed_t1.plus(a.times(delta_t));
                            pos_tPlusDelta1 = pos_t.plus(speed_t1.times(delta_t)).plus(a.times(delta_t*delta_t/2));
                        }
                    else{
                            speed_tPlusDelta1 = speed_tPlusDelta;
                            pos_tPlusDelta1 = pos_tPlusDelta;
                    }

                }
              }
       }
   }
   /**
    * Metodo para obtener la descripcion del bloque.
    * @return retorno del string identificador del bloque.
    */
   public String getDescription() {
      return "Block#" + getId() + ": "+getPosition().getDescription()+",";
   }
   /**
    * Metodo para obtener el estado del bloque.
    * @return retorno de la posicion del bloque.
    */
   public String getState() {
         return getPosition()+",";
   }
} 
