/**
 * Clase del objeto Cuerda.
 * @author Luis Fuentes / Benjamin Ginouves
 */
public class MyString extends PhysicsElement{
   private static int id=0;  // Spring identification.
   protected float restLength; // Largo del a cuerda.
   protected final float stiffness; // Coeficiente de elasticidad de la cuerda.
   protected Block block_a, block_b; // bloques en los extremos A y B respectivamente.
   private MyString(){   // nobody can create a block without state
      super(id++);
      stiffness=0;
   }
   /**
    * Implementacion y especificacion de los atributos del objeto cuerda.
    * @param restLength largo de la cuerda.
    * @param stiffness coeficiente de elasticidad, este debe ser mayor a 1000 para simular correctamente una cuerda.
    */
   public MyString(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      block_a = block_b = null;
   }
   /**
    * Metodo para vincular el extremo A al bloque.
    * @param a bloque a vincular.
    */
   public void attachBlock_a (Block a) {
      block_a = a;
      block_a.attachString(this);
   }
   /**
    * Metodo para remover del extremo A el bloque.
    */
   public void detachBlock_a(){
      if (block_a == null) return;
      block_a.detachString(this);
      block_a=null;
   }
   /**
    * Metodo para vincular el extremo B al bloque.
    * @param b bloque a vincular.
    */
   public void attachBlock_b (Block b) {
      block_b = b;
      block_b.attachString(this);
   }
   /**
    * Metodo para remover del extremo B el bloque.
    */
   public void detachBlock_b(){
      if (block_b == null) return;
      block_b.detachString(this);
      block_b=null;
   }
   /**
    * Metodo para obtener el vector de posicion del extremo A de la cuerda si tiene vinculado un bloque.
    * @return vector de posicion.
    */
   protected Vector2D getPositionA() {
      if (block_a != null)
         return block_a.getPosition();
      return null;
   }
   /**
    * Metodo para obtener el vector de posicion del extremo B de la cuerda si tiene vinculado un bloque.
    * @return vector de posicion.
    */
   protected Vector2D getPositionB() {
      if (block_b != null)
         return block_b.getPosition();
      return null;
   }
   /**
    * Metodo para obtener el vector que describe la distancia entre los extremos de la cuerda.
    * @return vector.
    */
   protected Vector2D getVector() {
      Vector2D b = getPositionB();
      Vector2D a = getPositionA();
      if (a== null || b==null) return new Vector2D();
      else return b.minus(a);
   }
   /**
    * Metodo para obtener la fuerza que ejerce la cuerda sobre el bloque conectado en su extremo.
    * @param block bloque conectado al extremo de la cuerda.
    * @return vector.
    */
   public Vector2D getForce(Block block) {
      Vector2D force;
      if ((block_a == null) || (block_b == null))
         return new Vector2D();
      Vector2D stringVector = getVector();
      double stretch = stringVector.module() - restLength;
      if(stretch > 0.1*restLength)
          force = stringVector.unitary().times(stiffness*0.1*restLength);
      else if(stretch > 0)
          force = stringVector.unitary().times(stretch*stiffness);
      else
          force = new Vector2D();
      if (block == block_a) return force;
      if (block == block_b) return force.times(-1);
      else return new Vector2D();
   }
   /**
    * Metodo para obtener el estiramiento de la cuerda
    * @return largo
    */
   protected double getStretch() {
      if ((block_a == null) || (block_b == null))
         return 0;
      Vector2D stringVector = getVector();
      return (stringVector.module() - restLength);
   }
   /**
    * Metodo no utilizado.
    * @param delta_t
    * @param gravity
    */
   public void computeNextState(double delta_t, double gravity) {
   }
   /**
    * Metodo no utilizado.
    * @param delta_t
    */
   public void checkNext(double delta_t){
   }
   /**
    * Metodo no utilizado.
    * @param damp
    */
   public void updateState(boolean damp){
   }
   /**
    * Metodo para obtener la informacion de la cuerda.
    * @return descripcion.
    */
   public String getDescription() {
      return "Spring#"+ getId()+": "+Vector2D.getDescription()+
            ","+Vector2D.getDescription()+",";
   }
   /**
    * Metodo para obtener el estiramiento en el proximo estado de la cuerda.
    * @return estiramiento.
    */
   public double nextStretch(){
      if ((block_a == null) || (block_b == null))
         return 0;
      Vector2D stringVector = nextVector();
      return (stringVector.module() - restLength);
   }
   /**
    * Metodo para obtener el vector que describe el largo de la cuerda en el siguiente estado.
    * @return vector.
    */
   protected Vector2D nextVector() {
      Vector2D b = nextPositionB();
      Vector2D a = nextPositionA();
      if (a== null || b==null) return new Vector2D();
      else return b.minus(a);
   }
   /**
    * Metodo para obtener la siguiente posicion del extremo A de la cuerda si tiene vinculado un bloque.
    * @return vector de posicion.
    */
   protected Vector2D nextPositionA() {
      if (block_a != null)
         return block_a.nextPosition();
      return null;
   }
   /**
    * Metodo para obtener la siguiente posicion del extremo B de la cuerda si tiene vinculado un bloque
    * @return vector e posicion.
    */
   protected Vector2D nextPositionB() {
      if (block_b != null)
         return block_b.nextPosition();
      return null;
   }
   /**
    * Metodo para obtener la informacion de la posicion
    * @return String con la descripcion.
    */
   public String getState() {
      String s;
      if (block_a != null)
         s = block_a.getPosition()+",";
      else if (block_b != null)
              s = block_b.getPosition().minus(new Vector2D(restLength,0))+",";
            else
              s = new Vector2D()+",";  // up to here left end is determined
      if (block_a != null)
         s += block_a.getPosition();
      else if (block_b != null)
               s += block_b.getPosition().plus(new Vector2D(restLength,0));
            else
               s += new Vector2D();
      return s+",";
   }
}
